#!/usr/bin/env python3
# -*- coding: utf-8 -*-
'''
Script to produce Figure 2 and 3 in Weber et al. (2018) in Journal of Geophysical Research - Earth Surface 

:copyright:
    Samuel Weber (samuel.weber@geo.uzh.ch)
:version:
    April 10, 2018
'''

import socket
if socket.gethostname() == 'tik42x' or socket.gethostname() == 'tik43x':
    import matplotlib as mpl
    mpl.use('Agg')

import numpy as np
import pandas as pd
import datetime as dt
import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.gridspec as gridspec
import matplotlib.transforms as mtransforms
timestamp2datetime = np.vectorize(dt.datetime.utcfromtimestamp)

from obspy.core import UTCDateTime
from obspy.imaging.cm import obspy_sequential
from toolbox import get_target_path, get_source_path, get_vault_path, import_miniseed_data, conv_data2velocity, spectrogram, gen_df_waveform, detectCoincidence, yyyymmdd_hhmmss
from load_alldata import run_load_alldata, timeseries_aggregation, timeseries_drop_timeperiods, get_disturbance_dataavailability_MH
from trigger_events_miniseed import get_trigger_sensitivity_ums

    
import locale
locale.setlocale(locale.LC_ALL, 'C')

# Data path
PATH_DATA_SOURCE = get_source_path()
PATH_DATA_TARGET = get_target_path()
PATH_DATA_VAULT  = get_vault_path()


#%% LOAD MH DATA and REMOVE TRIGGER SENSITIVITY
# 5 - 15 August
tbeg_MH = dt.datetime(2015,8,8)
tend_MH = dt.datetime(2015,8,15)
WINDOW = 10

# 14 August 20:09 - 20:12
#tbeg_MH = dt.datetime(2015,8,14,20,9)
#tend_MH = dt.datetime(2015,8,14,20,12)
#WINDOW = 1

trng_MH = pd.date_range(tbeg_MH, tend_MH, freq='1min', utc=True)
trng_MH_hour = pd.date_range(tbeg_MH, tend_MH, freq='1h', utc=True)
trng_MH_min = pd.date_range(tbeg_MH, tend_MH, freq='1min', utc=True)


if 'MH02' not in locals():
    print('Data has to be loaded. This takes very long!')
    MH02, MH03, MH06, MH20, MH25, MH27, MH29low, MH29high, MH30, MH36_nofilter, MH38_nofilter, MH40, IMIS = run_load_alldata(data2hdf = False, LOAD = True, tbeg=tbeg_MH, tend=tend_MH, FILTER = 0)
    MH02, MH03, MH06, MH20, MH25, MH27, MH29low, MH29high, MH30, MH36_filtered, MH38_filtered, MH40, IMIS = run_load_alldata(data2hdf = False, LOAD = True, tbeg=tbeg_MH, tend=tend_MH, FILTER = 1)
    coincidence_MH36_nofilter, coincidence_MH38_nofilter = detectCoincidence(MH36_nofilter.events, MH38_nofilter.events, amp_level1 = 250, amp_level2 = 500, tol = 0.1)
    coincidence_MH36_filtered, coincidence_MH38_filtered = detectCoincidence(MH36_filtered.events, MH38_filtered.events, amp_level1 = 125, amp_level2 = 250, tol = 0.1)
    
    print('Unify data units...')
    print('... event\'s amp_max in nm/s')
    MH36_nofilter.events['amp_max_vel_nms']  = MH36_nofilter.events['amp_max_vel'].values * 10**9
    MH36_filtered.events['amp_max_vel_nms']  = MH36_filtered.events['amp_max_vel'].values * 10**9
    MH38_nofilter.events['amp_max_vel_nms']  = MH38_nofilter.events['amp_max_vel'].values * 10**9
    MH38_filtered.events['amp_max_vel_nms']  = MH38_filtered.events['amp_max_vel'].values * 10**9
    MH40.events['amp_max_vel_nms']           = MH40.events['amp_max_vel'].values * 10**9
    
    print('... event\'s energy in nm/s')
    MH36_nofilter.events['energy_vel_nms']  = MH36_nofilter.events['energy_vel'].values * 10**18
    MH36_filtered.events['energy_vel_nms']  = MH36_filtered.events['energy_vel'].values * 10**18
    MH38_nofilter.events['energy_vel_nms']  = MH38_nofilter.events['energy_vel'].values * 10**18
    MH38_filtered.events['energy_vel_nms']  = MH38_filtered.events['energy_vel'].values * 10**18
    MH40.events['energy_vel_nms']           = MH40.events['energy_vel'].values * 10**18
    
    print('ToDo: Convert amplitude from V or mV to velocity for AE!!!')
    MH29high.events['amp_max_vel_nms']  = MH29high.events.amplitude.values
    MH29low.events['amp_max_vel_nms']   = MH29low.events.amplitude.values
    MH29high.events['duration'] = MH29high.events['duration']/1000
    MH29low.events['duration']  = MH29low.events['duration']/1000
    MH29high.events['energy_vel_nms'] = MH29high.events['energy']
    MH29low.events['energy_vel_nms']  = MH29low.events['energy']
    
    print('Load irreversibility index')
    d = pd.read_csv(PATH_DATA_SOURCE + 'timeseries/mh02.index.csv',skiprows=0)
    d.time = pd.to_datetime(d.time,utc=True)
    index_MH02 = d.set_index('time')
    d = pd.read_csv(PATH_DATA_SOURCE + 'timeseries/mh03.index.csv',skiprows=0)
    d.time = pd.to_datetime(d.time,utc=True)
    index_MH03 = d.set_index('time')
    d = pd.read_csv(PATH_DATA_SOURCE + 'timeseries/mh08.index.csv',skiprows=0)
    d.time = pd.to_datetime(d.time,utc=True)
    index_MH08 = d.set_index('time')



ASSIGN_AGGREGATE = True
if ASSIGN_AGGREGATE:
    print('Assign variables and aggregate timeseries')
    T_E = timeseries_aggregation(MH27, aggregation_interval=1)
    T_E.interpolate('linear',limit=10,inplace=True)
    T_N = timeseries_aggregation(MH30, aggregation_interval=1)
    T_N.interpolate('linear',limit=10,inplace=True)
    
    triggersensitivity_MH36_nofilter, triggersensitivity_MH36_filtered, triggersensitivity_MH38_nofilter, triggersensitivity_MH38_filtered, triggersensitivity_MH40 = get_trigger_sensitivity_ums()
    
    AEhigh = MH29high.events.copy()
    AElow  = MH29low.events.copy()
    AM     = MH40.events.copy()
    SM1    = MH36_nofilter.events[MH36_nofilter.events.amp_max_vel_nms > triggersensitivity_MH36_nofilter * 1e3]
    SM1f   = MH36_filtered.events[MH36_filtered.events.amp_max_vel_nms > triggersensitivity_MH36_filtered * 1e3]
    SM2    = MH38_nofilter.events[MH38_nofilter.events.amp_max_vel_nms > triggersensitivity_MH38_nofilter * 1e3]
    SM2f   = MH38_filtered.events[MH38_filtered.events.amp_max_vel_nms > triggersensitivity_MH38_filtered * 1e3]

REMOVE_DISTURBANCE = True
disturbance_MH, dataavailability_MH = get_disturbance_dataavailability_MH(trng_MH)
dataavailability_MH_hour = timeseries_aggregation(dataavailability_MH,aggregation_interval=60)
if REMOVE_DISTURBANCE:
    print('Load disturbance and data availability timeseries')
    flags = np.convolve(1 * disturbance_MH.fieldvisit,[1,-1])[:-1]
    t1 = trng_MH[flags==1]
    t2 = trng_MH[flags==-1]
    
    AEhigh = timeseries_drop_timeperiods(AEhigh, t1, t2)
    AElow  = timeseries_drop_timeperiods(AElow, t1, t2)
    AM     = timeseries_drop_timeperiods(AM, t1, t2)
    SM1    = timeseries_drop_timeperiods(SM1, t1, t2)
    SM1f   = timeseries_drop_timeperiods(SM1f, t1, t2)
    SM2    = timeseries_drop_timeperiods(SM2, t1, t2)
    SM2f   = timeseries_drop_timeperiods(SM2f, t1, t2)

DROP_MOUNTAINEERS = False
if DROP_MOUNTAINEERS:
    print('Load mountaineer activity timeseries')
    mountaineer = disturbance_MH.mountaineers
    ind = mountaineer.activity
    flags = np.convolve(1 * ind,[1,-1])[:-1]
    t1 = trng_MH[flags==1]
    t2 = trng_MH[flags==-1]
    t1 = t1 - dt.timedelta(1/86400 * 60 * 5)
    t2 = t2 + dt.timedelta(1/86400 * 60  * 5)
    
    AEhigh = timeseries_drop_timeperiods(AEhigh, t1, t2)
    AElow  = timeseries_drop_timeperiods(AElow, t1, t2)
    AM     = timeseries_drop_timeperiods(AM, t1, t2)
    SM1    = timeseries_drop_timeperiods(SM1, t1, t2)
    SM1f   = timeseries_drop_timeperiods(SM1f, t1, t2)
    SM2    = timeseries_drop_timeperiods(SM2, t1, t2)
    SM2f   = timeseries_drop_timeperiods(SM2f, t1, t2)
    no_mountaineer = timeseries_drop_timeperiods(mountaineer,t1,t2)



AEhigh_bak = AEhigh.copy()
AElow_bak  = AElow.copy()
AM_bak     = AM.copy()
SM1_bak    = SM1.copy()
SM1f_bak   = SM1f.copy()
SM2_bak    = SM2.copy()
SM2f_bak   = SM2f.copy()


#%% FUNCTION TO LOAD DATA
def get_data(station, WINDOW, channel, dB=True):

    if WINDOW==10:
        SPECTRA = pd.read_hdf(PATH_DATA_VAULT + 'data_archive/deployments/matterhorn2007/analysis_AEMS/spectra/spectra_10sec/' + station + '_spectra_10sec.h5','spectra_10sec')
        SPECTRA.index = pd.to_datetime([time.datetime for time in SPECTRA.index.values], utc=True)
        trng_MH_10sec = pd.date_range(start=SPECTRA.index[0], end=SPECTRA.index[-1], freq='10s', utc=True)
        SPECTRA = SPECTRA.reindex(trng_MH_10sec)
    elif WINDOW==1:
        SPECTRA = pd.read_hdf(PATH_DATA_VAULT + 'data_archive/deployments/matterhorn2007/analysis_AEMS/spectra/spectra_1sec/' + station + '_spectra_1sec.h5','spectra_1sec')
        SPECTRA.index = pd.to_datetime([time.datetime for time in SPECTRA.index.values], utc=True)
        trng_MH_1sec = pd.date_range(start=SPECTRA.index[0], end=SPECTRA.index[-1], freq='1s', utc=True)
        SPECTRA = SPECTRA.reindex(trng_MH_1sec)
    elif WINDOW==500:
        SPECTRA = pd.read_hdf(PATH_DATA_VAULT + 'data_archive/deployments/matterhorn2007/analysis_AEMS/spectra/spectra_500msec/' + station + '_spectra_500msec.h5','spectra_500msec')
        SPECTRA.index = pd.to_datetime([time.datetime for time in SPECTRA.index.values], utc=True)
        trng_MH_500msec = pd.date_range(start=SPECTRA.index[0], end=SPECTRA.index[-1], freq='1s', utc=True)
        SPECTRA = SPECTRA.reindex(trng_MH_500msec)



    SPECTRA['fft_nan'] = SPECTRA['freq'] * np.NaN
    SPECTRA['fft_nan'].fillna(method='ffill', inplace=True)
    SPECTRA['wf_nan'] = SPECTRA['wf_Z'] * np.NaN
    SPECTRA['wf_nan'].fillna(method='ffill', inplace=True)
    for var in ['fft_Z', 'fft_N', 'fft_E']:
        SPECTRA[var].fillna(SPECTRA['fft_nan'],inplace=True)
    for var in ['wf_Z', 'wf_N', 'wf_E']:
        SPECTRA[var].fillna(SPECTRA['wf_nan'],inplace=True)
    
    time = SPECTRA.index.values 
    waveform = SPECTRA['wf_{:s}'.format(channel)].values
    
    if (SPECTRA['freq'].values[0][-1] == SPECTRA['freq'].values[-1][-1]):
        f = SPECTRA['freq'].values[0][1:]
    else:
        f = []
        print('Not every where the same frequency range')

    matrix = np.vstack(SPECTRA['fft_{:s}'.format(channel)].values)
    matrix = matrix.transpose()
    matrix = np.ma.masked_where(np.isnan(matrix),matrix)
    print(matrix.shape)
    if dB==True:
        matrix = 10 * np.log10(matrix[1:, :]) # Log scaling for amplitude values, factor 20 for dB
    else:
        matrix = np.sqrt(matrix[1:, :])
    S = matrix

    return time, waveform, f, S

# Define a hit column
for sensor in ['AEhigh','AElow','AM','SM1','SM1f','SM2','SM2f']:
    print('Hits are defined for ' + sensor)
    eval(sensor)['hits'] = eval(sensor)['energy_vel_nms'] * 0 + 1

#%% OVERVIEW FIGURE 1, with pre-calculated spectrograms

tbeg_MH = dt.datetime(2015,8,7)
tend_MH = dt.datetime(2015,8,15)
trng_MH_1sec = pd.date_range(start=tbeg_MH, end=tend_MH, freq='1s', utc=True)
WINDOW = 10

TBEG=UTCDateTime(tbeg_MH)
TEND=UTCDateTime(tend_MH)
t1 = TBEG -1
t2 = TEND +1

dB = True
fig = plt.figure(figsize=(24/2.54, 24/2.54), facecolor='w', edgecolor='k')
gs = gridspec.GridSpec(6, 1, height_ratios=[1,1,1,1,1,1], left=0.09, right=0.99, bottom=0.06, top=0.97, wspace = 0.06, hspace=0.4) # left=0.125, right=0.9, bottom=0.1, top=0.9, wspace=0.2, hspace=0.2
gs0 = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec = gs[0,0], hspace = 0.2)
gs1a = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec = gs[1,0], hspace = 0.2)
gs1b = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec = gs[2,0], hspace = 0.2)
gs2 = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec = gs[3,0], hspace = 0)
gs3 = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec = gs[4,0], hspace = 0)
gs4 = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec = gs[5,0], hspace = 0)
ax = {}
xpos_ylabel = -0.07

print('... crackmeter data')
ax[0] = plt.subplot(gs0[0])
ax[0].plot(MH02.dx0,'-', label='measured',color='blue', zorder=2) #,linewidth=1
ax[0].plot([MH02.index[-1], MH02.index[-1] + dt.timedelta(1/24/60*2)], [MH02.dx0[-1], MH02.dx0[-1] + 1],ls='--',color='blue',label='break off', zorder=2)
ax[0].set_title('Fracture displacement measured at $\mathregular{CR}_\mathregular{old}$ (accuracy $< 0.01 \, \mathregular{mm}$)', fontsize=10, fontweight='bold',ha='center')
ax[0].legend(numpoints=1,frameon=False, loc="upper left", ncol=2, fontsize=10, handletextpad=0.2, handlelength=1.4, bbox_to_anchor=(0,1.1))
ax[0].set_ylabel('mm')
ax[0].yaxis.set_label_coords(xpos_ylabel, 0)
ax[0].set_ylim([42.4,42.9])
[label.set_visible(False) for label in ax[0].get_xticklabels()]
ax[0].tick_params(axis='x', colors='white')
ax[10] = plt.subplot(gs0[1],sharex=ax[0])
ax[10].plot(MH02.dx0,'-', label='$\mathregular{CR}_\mathregular{02}$',color='blue', zorder=2)
ax[10].set_ylim([29.1,29.6])
[label.set_visible(False) for label in ax[10].get_xticklabels()]

print('... AE')
ax[1] = plt.subplot(gs1a[0], sharex=ax[0])
[label.set_visible(False) for label in ax[1].get_xticklabels()]
values = timeseries_aggregation(AEhigh.hits,aggregation_interval=1/60,function='sum').reindex(trng_MH_1sec)
values[np.isnan(values)] = 0
cumulative = np.cumsum(values)
ax[1].plot(cumulative,label='cumulative',color='blue', zorder=2)
ax[1].legend(numpoints=1,frameon=False, loc="upper left", ncol=1, fontsize=10, handletextpad=0.2, handlelength=1.4, bbox_to_anchor=(0,1.05))

ax[1].set_title('Acoustic emission hits captured by $\mathregular{AS}_\mathregular{scarp,high}$ (sensitive $35-100\,\mathregular{kHz}$)', fontsize=10, fontweight='bold',ha='center')
ax[1].set_ylabel('hits')
ax[1].yaxis.set_label_coords(xpos_ylabel, 0.5)

ax[11] = plt.subplot(gs1b[0],sharex=ax[0])
[label.set_visible(False) for label in ax[11].get_xticklabels()]
values = timeseries_aggregation(AElow.hits,aggregation_interval=1/60,function='sum').reindex(trng_MH_1sec)
values[np.isnan(values)] = 0
cumulative = np.cumsum(values)
ax[11].plot(cumulative,label='cumulative',color='blue', zorder=2)
ax[11].legend(numpoints=1,frameon=False, loc="upper left", ncol=1, fontsize=10, handletextpad=0.2, handlelength=1.4, bbox_to_anchor=(0,1.05))

ax[11].set_title('Acoustic emission hits captured by $\mathregular{AS}_\mathregular{scarp,low}$ (sensitive $5-30\,\mathregular{kHz}$)', fontsize=10, fontweight='bold',ha='center')
ax[11].set_ylabel('hits')
ax[11].yaxis.set_label_coords(xpos_ylabel, 0.5)

print('... AM')
time, waveform, f, S = get_data('MH40', WINDOW, 'Z', dB=True)
norm_factor = np.nanmax(np.abs(np.concatenate(waveform[(time > np.datetime64(tbeg_MH)) & (time < np.datetime64(tend_MH))])))
norm_factor = 1e-6

ax[2] = plt.subplot(gs2[0], sharex=ax[0])
ax[2].plot(np.repeat(time,2), np.concatenate(waveform)/norm_factor,label='vertical component',color='black', alpha=0.7, linewidth=1, zorder=2)
[label.set_visible(False) for label in ax[2].get_xticklabels()]
ax[2].tick_params(axis='x', colors='white')

ax[2].set_title('Signal captured by accelerometer $\mathregular{AM}_\mathregular{scarp}$ (sensitive $10-10^4\,\mathregular{Hz}$)', fontsize=10, fontweight='bold',ha='center')
ax[2].set_ylabel('$\mathregular{\mu m \ s^{-1}}$')
ax[2].yaxis.set_label_coords(xpos_ylabel, 0.5)
ax[2].legend(numpoints=1,frameon=False, loc="upper left", ncol=1, fontsize=10, handletextpad=0.2, handlelength=1.4, bbox_to_anchor=(0,1.05))

print('... SM scarp')
time, waveform, f, S = get_data('MH36', WINDOW, 'Z', dB=True)
try:
    norm_factor = np.nanmax(np.abs(np.concatenate(waveform[(time > np.datetime64(tbeg_MH)) & (time < np.datetime64(tend_MH))])))
except:
    norm_factor = 1
    pass
norm_factor = 1e-6

ax[3] = plt.subplot(gs3[0], sharex=ax[0])
ax[3].plot(np.repeat(time,2), np.concatenate(waveform)/norm_factor,label='vertical component',color='black', alpha=0.7, linewidth=1, zorder=2)
[label.set_visible(False) for label in ax[3].get_xticklabels()]
ax[3].tick_params(axis='x', colors='white')

ax[3].set_title('Signal captured by seismometer $\mathregular{SM}_\mathregular{scarp}$ (sensitive $1-10^2\,\mathregular{Hz}$)', fontsize=10, fontweight='bold',ha='center')
ax[3].set_ylabel('$\mathregular{\mu m \ s^{-1}}$')
ax[3].yaxis.set_label_coords(xpos_ylabel, 0.5)
ax[3].legend(numpoints=1,frameon=False, loc="upper left", ncol=1, fontsize=10, handletextpad=0.2, handlelength=1.4, bbox_to_anchor=(0,1.05))

print('... SM ridge')
time, waveform, f, S = get_data('MH38', WINDOW, 'Z', dB=True)
norm_factor = np.nanmax(np.abs(np.concatenate(waveform[(time > np.datetime64(tbeg_MH)) & (time < np.datetime64(tend_MH))])))
norm_factor = 1e-6

ax[4] = plt.subplot(gs4[0], sharex=ax[0])
ax[4].plot(np.repeat(time,2), np.concatenate(waveform)/norm_factor,label='vertical component',color='black', alpha=0.7, linewidth=1, zorder=2)

ax[4].set_title('Signal captured by seismometer $\mathregular{SM}_\mathregular{ridge}$ (sensitive $1-10^2\,\mathregular{Hz}$)', fontsize=10, fontweight='bold',ha='center')
ax[4].set_ylabel('$\mathregular{\mu m \ s^{-1}}$')
ax[4].yaxis.set_label_coords(xpos_ylabel, 0.5)
ax[4].legend(numpoints=1,frameon=False, loc="upper left", ncol=1, fontsize=10, handletextpad=0.2, handlelength=1.4, bbox_to_anchor=(0,1.05))
   
ax[2].set_ylim([-40,40])
ax[2].set_yticks([-20,0,20])
for i in [3,4]:
    ax[i].set_ylim([-300,300])
    ax[i].set_yticks([-150,0,150])


for i in [0,10,1,11,2,3,4]:
    ax[i].axvline(x=dt.datetime(2015,8,14,20,10,15), ymin=0, ymax=1, linewidth=2, color = 'r',alpha=0.6, zorder=1)

for i in [10,1,11,2,3,4]:    
    ax[i].axvline(x=dt.datetime(2015,8,14,9,40,40), ymin=0, ymax=1, linewidth=2, color = 'gray',alpha=0.6, zorder=1)
    
    trans = mtransforms.blended_transform_factory(ax[i].transData, ax[i].transAxes)
    ax[i].fill_between([dt.datetime(2015,8,9,20,30),dt.datetime(2015,8,9,22,30)], 0,1, facecolor='gray',alpha=0.6, transform=trans, zorder=1)

ax[10].annotate('fracture', xy=(dt.datetime(2015,8,9,20,30), 29.375), xytext=(dt.datetime(2015,8,9,12), 29.425), ha='right',
            arrowprops=dict(arrowstyle="->"))
ax[0].annotate('fracture', xy=(dt.datetime(2015,8,14,9,40,40), 42.43), xytext=(dt.datetime(2015,8,14,1,40), 42.555), ha='right',
            arrowprops=dict(arrowstyle="->"))
ax[0].annotate('failure', xy=(dt.datetime(2015,8,14,20,10,32), 42.6), xytext=(dt.datetime(2015,8,14,12,10), 42.725), ha='right',
            arrowprops=dict(arrowstyle="->"))


ax[0].set_xlim([tbeg_MH,tend_MH])
if WINDOW==10:
    date_format = mdates.DateFormatter('%d.%m.%Y')
elif WINDOW==1:
    date_format = mdates.DateFormatter('%H:%M:%S')

ax[0].xaxis.set_major_formatter(date_format)
if WINDOW==10:
    major = mdates.DayLocator(bymonthday=range(0,32,2))#,interval=2)
    minor = mdates.DayLocator(interval=1)
    ax[0].xaxis.set_major_locator(major)
    ax[0].xaxis.set_minor_locator(minor)

ax[4].set_xlabel('Date (DD.MM.YYYY, UTC)')


fname = 'breakoff_wf__summary0_' + yyyymmdd_hhmmss(TBEG) + '-' + yyyymmdd_hhmmss(TEND)
plt.savefig(PATH_DATA_TARGET + fname + '.png', dpi=300)
print('Figure is saved as ' + fname)



#%% OVERVIEW FIGURE 1B, with realtime-calculated spectrograms

fig = plt.figure(figsize=(24/2.54, 12/2.54), facecolor='w', edgecolor='k')
gs = gridspec.GridSpec(2, 1, left=0.08, right=0.9, bottom=0.13, top=0.92, wspace = 0.06, hspace=0.35) #DEFAULT: left=0.125, right=0.9, bottom=0.1, top=0.9, wspace=0.2, hspace=0.2
gs1 = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec = gs[0,0], hspace = 0)
gs2 = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec = gs[1,0], hspace = 0)
ax = {}
xpos_ylabel = -0.07


WINDOW = 0.5
OVERLAP = 0.9
dB = True

tbeg = dt.datetime(2015,8,14,20,10,15)
tend = dt.datetime(2015,8,14,20,11,15)

TBEG = UTCDateTime(tbeg)
TEND = UTCDateTime(tend)

t1 = TBEG-10
t2 = TEND+10

stream_SM = import_miniseed_data([t1,t2],'MH38',['EHZ'])
st = stream_SM.copy()
st = conv_data2velocity(st)
#st = filter_stream(st, FREQMIN=1, FREQMAX=100)
st = st.slice(t1,t2)
trace_SM = st[0]

stream_AM = import_miniseed_data([t1,t2],'MH40',['EHZ'])
st = stream_AM.copy()
st = conv_data2velocity(st)
#st = filter_stream(st, FREQMIN=10, FREQMAX=6000)
st = st.slice(t1,t2)
trace_AM = st[0]

    
tr = trace_AM.copy()
ax[1] = plt.subplot(gs1[0])
waveform = gen_df_waveform(tr, t1, t2) * 1e6

ax[1].plot(waveform,label='vertical component',color='black', alpha=0.7, linewidth=1)
[label.set_visible(False) for label in ax[1].get_xticklabels()]
ax[11] = plt.subplot(gs1[1], sharex=ax[1])
S, f, time, norm = spectrogram(tr, freq_lim=[10,6000], window=WINDOW, overlap=OVERLAP, dB=dB)
im = ax[11].pcolormesh(time, f, S, norm=norm, cmap=obspy_sequential)
[label.set_visible(False) for label in ax[11].get_xticklabels()]
ax[11].set_yscale('log')

ax[1].set_ylim([-4,4])
ax[1].set_yticks([-2,0,2])


ax[1].set_title('Signal captured by accelerometer $\mathregular{AM}_\mathregular{scarp}$ (sensitive $10-10^4\,\mathregular{Hz}$)', fontsize=10, fontweight='bold',ha='center')
ax[1].set_ylabel('$\mathregular{\mu m \ s^{-1}}$')
ax[1].yaxis.set_label_coords(xpos_ylabel, 0.5)
ax[11].set_ylabel('$\mathregular{Hz}$')
ax[11].yaxis.set_label_coords(xpos_ylabel, 0.5)
ax[1].legend(numpoints=1,frameon=False, loc="upper right", ncol=1, fontsize=10, handletextpad=0.2, handlelength=1.4, bbox_to_anchor=(1,1.05))

im.set_clim([-230,-110])

tr = trace_SM.copy()
ax[2] = plt.subplot(gs2[0],sharex=ax[1])
waveform = gen_df_waveform(tr, t1, t2) * 1e6

ax[2].plot(waveform,label='vertical component',color='black', alpha=0.7, linewidth=1)
[label.set_visible(False) for label in ax[2].get_xticklabels()]
ax[12] = plt.subplot(gs2[1],sharex=ax[1])
S, f, time, norm = spectrogram(tr, freq_lim=[1,100], window=WINDOW, overlap=OVERLAP, dB=dB)
im2 = ax[12].pcolormesh(time, f, S, norm=norm, cmap=obspy_sequential)
[label.set_visible(True) for label in ax[12].get_xticklabels()]
ax[12].set_yticks([25,50,75])


ax[2].set_ylim([-300,300])
ax[2].set_yticks([-150,0,150])


ax[2].set_title('Signal captured by seismometer $\mathregular{SM}_\mathregular{ridge}$ (sensitive $1-10^2\,\mathregular{Hz}$)', fontsize=10, fontweight='bold',ha='center')
ax[2].set_ylabel('$\mathregular{\mu m \ s^{-1}}$')
ax[2].yaxis.set_label_coords(xpos_ylabel, 0.5)
ax[12].set_ylabel('$\mathregular{Hz}$')
ax[12].yaxis.set_label_coords(xpos_ylabel, 0.5)
ax[2].legend(numpoints=1,frameon=False, loc="upper right", ncol=1, fontsize=10, handletextpad=0.2, handlelength=1.4, bbox_to_anchor=(1,1.05))


# Add the colorbar outside for both subplots
box1 = ax[1].get_position()
box11 = ax[11].get_position()
box2 = ax[2].get_position()
box12 = ax[12].get_position()
pad, width = 0.02, 0.02
cax = fig.add_axes([box12.xmax + pad, box12.ymin, width, (box1.ymax-box12.ymin)*0.96])
cbar = fig.colorbar(im2, cax=cax)
cbar.ax.set_title('PSD (dB)', fontsize=10,x=2.15)
im2.set_clim([-230,-110])
cbar.set_ticks(np.arange(-230,-100,20)[1:-1])

ax[12].set_xlim([tbeg,tend])

major = mdates.SecondLocator(interval=15)
minor = mdates.SecondLocator(interval=5)
ax[12].xaxis.set_major_locator(major)
ax[12].xaxis.set_minor_locator(minor)

ticks = ax[12].get_xticks()
ax[12].set_xticks(ticks[1:-1])

date_format = mdates.DateFormatter('%H:%M:%S')
ax[12].xaxis.set_major_formatter(date_format)

ax[12].set_xlabel('Time (HH:MM:SS, UTC)')

fname = 'breakoff_wf__summary00_' + yyyymmdd_hhmmss(TBEG) + '-' + yyyymmdd_hhmmss(TEND)
plt.savefig(PATH_DATA_TARGET + fname + '.png', dpi=300)
print('Figure is saved as ' + fname)

